#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    // BGG audio stuff
    int sr = 44100;
    int nbufs = 2; // you can use more for more processing but latency will suffer
    
    nchans = 2;
    framesize = 512;
    
    s_audio_outbuf = (short*)malloc(nchans*framesize*sizeof(short));
    
    // initialize RTcmix
	rtcmixmain();
	maxmsp_rtsetparams(sr, nchans, framesize, NULL, NULL);
    
    ofSoundStreamSetup(nchans, 0, sr, framesize, nbufs);
    ofSoundStreamStart();

    
    // BGG music stuff
    char *thescore = { " \
        bus_config(\"WAVETABLE\", \"aux 0 out\") \
        bus_config(\"MOOGVCF\", \"aux 0 in\", \"aux 1-2 out\") \
        bus_config(\"PANECHO\", \"aux 1-2 in\", \"out 0-1\") \
        wave = maketable(\"wave\", 1000, \"saw\") \
        \
        ffreq = maketable(\"line\", \"nonorm\", 1000, 0, 7000, 0.5, 100) \
        ffmult = 0.7 \
        \
        PANECHO(0, 0, 999, 1, 0.125, 0.5, 0.05, 1, 0) \
        "};
    parse_score(thescore, strlen(thescore));
    
    is_running = 0;
    
    
    // BGG graphics stuff
    ofBackground(196, 216, 226);
    
    WIDTH = ofGetWidth();
    HEIGHT = ofGetHeight();
    
    theButton.setup(0.4*WIDTH, 0.2*HEIGHT, 0.2*WIDTH, 0.05*HEIGHT, WIDTH, HEIGHT);
}

void ofApp::audioRequested(float * output, int bufferSize, int nChannels) {
    pullTraverse(NULL, s_audio_outbuf);
    
    for (int i = 0; i < framesize*nchans; i++)
        output[i] = (float)s_audio_outbuf[i]/32786.0; // transfer to the float *output buf
    
    
    if (check_bang() == 1) {
        if (theButton.checkstate() == 1) {
            char *thescore = { " \
                dur = 0.5 \
                WAVETABLE(0, dur/2, 10000, 8.00, 0, wave) \
                WAVETABLE(0, dur/2, 10000, 8.001, 0, wave) \
                \
                MOOGVCF(0, 0, dur/2, 1, 0, 0.5, 0, ffreq*ffmult, 0.9) \
                \
                WAVETABLE(dur/2, dur/2, 10000, 8.00, 0, wave) \
                WAVETABLE(dur/2, dur/2, 10000, 8.001, 0, wave) \
                \
                MOOGVCF(dur/2, 0, dur/2, 1, 0, 0.5, 0, ffreq/2*ffmult, 0.9) \
                \
                MAXBANG(dur) \
                "};
            parse_score(thescore, strlen(thescore));
        }
    }
    
    char *pbuf = get_print();
    char *pbufptr = pbuf;
    while (strlen(pbufptr) > 0) {
        ofLog() << pbufptr;
        pbufptr += (strlen(pbufptr) + 1);
    }
    reset_print();
}


//--------------------------------------------------------------
void ofApp::update(){

}

//--------------------------------------------------------------
void ofApp::draw(){
    theButton.draw();

}

//--------------------------------------------------------------
void ofApp::exit(){

}

//--------------------------------------------------------------
void ofApp::touchDown(ofTouchEventArgs & touch){
    theButton.checkclick(touch.x, touch.y);
    if (theButton.checkstate() == 1) {
        if (is_running == 0) {
            char *thescore = { "MAXBANG(0)" };
            parse_score(thescore, strlen(thescore));
            is_running = 1;
        }
    }
    if (theButton.checkstate() == 0) is_running = 0;
}

//--------------------------------------------------------------
void ofApp::touchMoved(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchUp(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchDoubleTap(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchCancelled(ofTouchEventArgs & touch){
    
}

//--------------------------------------------------------------
void ofApp::lostFocus(){

}

//--------------------------------------------------------------
void ofApp::gotFocus(){

}

//--------------------------------------------------------------
void ofApp::gotMemoryWarning(){

}

//--------------------------------------------------------------
void ofApp::deviceOrientationChanged(int newOrientation){

}

